
CuiFinder User Options: 
   --config option set


UMLS-Interface Configuration Information
  Sources (SAB):
    SNOMEDCT
  Relations (REL):
    CHD
    PAR
  Database: 
    umls (MMSYS-2010AA-20100427)




The related (rb) CUIs to hand (C0018563):
  entire upper limb (body structure) (C1269078)
The related (rb) CUIs to hand (C1281583):
  entire upper limb (body structure) (C1269078)
